@@ -26,7 +26,7 @@
 
 __authors__ = ["J. Garriga"]
 __license__ = "MIT"
-__date__ = "22/12/2020"
+__date__ = "09/07/2021"
 
 import copy
 import glob
@@ -46,14 +46,14 @@ from silx.io import utils
 from silx.io import fabioh5
 from silx.io.url import DataUrl
 
-from darfix.core.dimension import AcquisitionDims, Dimension
+from darfix.core.dimension import AcquisitionDims, Dimension, POSITIONER_METADATA
 from darfix.core.imageOperations import img2img_mean, chunk_image
 from darfix.core.imageOperations import background_subtraction, background_subtraction_2D
 from darfix.core.imageOperations import hot_pixel_removal_3D, hot_pixel_removal_2D
 from darfix.core.imageOperations import threshold_removal
 from darfix.core.imageOperations import Method
 from darfix.core.imageRegistration import shift_detection, apply_shift
-from darfix.core.mapping import fit_data, compute_moments
+from darfix.core.mapping import fit_data, compute_moments, compute_rsm, compute_magnification
 from darfix.core.roi import apply_2D_ROI, apply_3D_ROI
 from darfix.core.utils import wrapTo2pi
 from darfix.decomposition.ipca import IPCA
@@ -92,26 +92,36 @@ class Dataset():
     :type raw_folder: Union[None,str], optional
     :param filenames: Ordered list of filenames, defaults to None.
     :type filenames: Union[Generator,Iterator,List], optional
-    :param treated: Name of the folder with the treated data, defaults 'treated_data'
-    :type treated: str, optional
+    :param dims: Dimensions dictionary
+    :type dims: AcquisitionDims, optional
+    :param transformation: Axes to use when displaying the images
+    :type transformation: ndarray, optional
     :param in_memory: If True, data is loaded into memory, else, data is read in
         chunks depending on the algorithm to apply, defaults to False.
     :type in_memory: bool, optional
+    :param copy_files: If True, creates a new treated data folder and doesn't replace
+        the directory files.
+    :type copy_files: bool, optional
     """
 
     def __init__(self, _dir, data=None, first_filename=None, filenames=None,
-                 dims=None, treated="treated_data", in_memory=True):
+                 dims=None, transformation=None, in_memory=True, copy_files=False):
 
         self._data = None
         self._frames_intensity = []
-        self._dir = _dir
-        self._treated = treated
         self._lock = threading.Lock()
         self.running_data = None
         self.moments_dims = {}
         self.operations_state = numpy.zeros(len(Operation))
-        if not os.path.isdir(self.dir + "/" + treated):
-            os.mkdir(self.dir + "/" + treated)
+        self._dir = _dir
+        self._transformation = transformation
+        if copy_files:
+            self._dir += "/treated"
+            if not os.path.isdir(self._dir):
+                try:
+                    os.mkdir(self._dir)
+                except PermissionError:
+                    raise PermissionError("Add a directory in the Treated Data tab with WRITE permission")
 
         if dims is None:
             self.__dims = AcquisitionDims()
@@ -126,7 +136,7 @@ class Dataset():
             self._data = data
         else:
             if filenames is None and first_filename is None:
-                filenames = sorted([x for x in glob.glob(self.dir + "/*") if os.path.isfile(x)])
+                filenames = sorted([x for x in glob.glob(_dir + "/*") if os.path.isfile(x)])
             self.filenames = filenames
             self.first_filename = first_filename
 
@@ -160,6 +170,14 @@ class Dataset():
         if self.running_data is not None:
             self.running_data.stop_operation(operation)
 
+    @property
+    def transformation(self):
+        return self._transformation
+
+    @transformation.setter
+    def transformation(self, value):
+        self._transformation = value
+
     @property
     def dir(self):
         return self._dir
@@ -272,6 +290,8 @@ class Dataset():
             return data.flatten()
         else:
             data = self.data.flatten()
+            if return_indices:
+                return (data, numpy.arange(len(data))) if indices is None else (data[indices], indices)
             return data if indices is None else data[indices]
 
     @property
@@ -332,7 +352,6 @@ class Dataset():
         self.__dims.remove_dim(axis)
 
     def zsum(self, indices=None, dimension=None):
-
         data = self.get_data(indices, dimension)
         return data.sum(axis=0)
 
@@ -392,6 +411,10 @@ class Dataset():
             _logger.info("Dimension {} of size {} has been added for reshaping"
                          .format(dimension.name, dimension.size))
 
+    def get_metadata_values(self, kind, key):
+        return numpy.unique([data.get_value(kind=kind, name=key)[0] for data
+                             in self.get_data().metadata])
+
     def __compute_changing_value(self, values, changing_value=1):
         """
         Recursive method used to calculate how fast is a dimension. The speed of a dimension is the number of
@@ -419,7 +442,7 @@ class Dataset():
                 self._dimensions_values[dimension[1].name] = values
         return self._dimensions_values
 
-    def compute_mosaicity_colorkey(self):
+    def compute_mosaicity_colorkey(self, scale=100):
         """
         Computes a mosaicity colorkey from the dimensions, and returns also the
         orientation distribution image.
@@ -430,8 +453,9 @@ class Dataset():
             new_shape = []
             dims_list = []
             for axis, dim in self.__dims:
-                dims_list += [numpy.linspace(-1, 1, dim.size)]
+                dims_list += [numpy.linspace(-1, 1, (dim.size - 1) * scale)]
                 new_shape += [dim.size]
+
             dims_list.reverse()
             mesh = numpy.meshgrid(*dims_list)
             if len(mesh) == 2:
@@ -441,11 +465,11 @@ class Dataset():
                     mesh[0], 2) + numpy.power(mesh[1], 2)) / numpy.sqrt(2)
                 hsv_key = numpy.stack((normalized_data, sqr, numpy.ones(
                     (len(dims_list[1]), len(dims_list[0])))), axis=2)
-                return ori_dist.reshape(new_shape), hsv_key
+                return ori_dist.reshape(numpy.flip(new_shape)).T, hsv_key
         return None, None
 
     def apply_background_subtraction(self, background=None, method="median", indices=None,
-                                     step=None, chunk_shape=[100, 100]):
+                                     step=None, chunk_shape=[100, 100], _dir=None):
         """
         Applies background subtraction to the data and saves the new data
         into disk.
@@ -471,9 +495,12 @@ class Dataset():
             the new urls.
         :rtype: Dataset
         """
-        bs_dir = self.dir + "/" + self.treated + "/bs/"
-        if not os.path.isdir(bs_dir):
-            os.mkdir(bs_dir)
+
+        _dir = self.dir if _dir is None else _dir
+
+        if not os.path.isdir(_dir):
+            os.mkdir(_dir)
+
         method = Method.from_value(method)
 
         self.running_data = self.get_data(indices)
@@ -497,7 +524,7 @@ class Dataset():
 
         if self._in_memory:
             new_data = background_subtraction(self.running_data, bg_data, method).view(Data)
-            new_data.save(bs_dir + "/data_bs")
+            new_data.save(_dir + "/data.hdf5")
             urls = new_data.urls
         else:
             bg = numpy.zeros(self.running_data[0].shape, self.running_data.dtype)
@@ -550,8 +577,12 @@ class Dataset():
                                 start[1] = chunk_shape[1] * (j + 1)
                             start[0] = chunk_shape[0] * (i + 1)
                             start[1] = 0
+
+            if not self.operations_state[Operation.BS]:
+                return
+
             urls = self.running_data.apply_funcs([(background_subtraction_2D, [bg])],
-                                                 save=bs_dir + "/data_bs", text="Applying background subtraction",
+                                                 save=_dir + "/data.hdf5", text="Applying background subtraction",
                                                  operation=Operation.BS)
             if urls is None:
                 return
@@ -570,9 +601,10 @@ class Dataset():
             new_data = Data(urls.reshape(self.data.urls.shape), self.data.metadata,
                             self._in_memory)
 
-        return Dataset(_dir=bs_dir, data=new_data, dims=self.__dims, in_memory=self._in_memory)
+        return Dataset(_dir=_dir, data=new_data, dims=self.__dims, transformation=self.transformation,
+                       in_memory=self._in_memory)
 
-    def apply_hot_pixel_removal(self, kernel=3, indices=None):
+    def apply_hot_pixel_removal(self, kernel=3, indices=None, _dir=None):
         """
         Applies hot pixel removal to Data, and saves the new data
         into disk.
@@ -590,17 +622,18 @@ class Dataset():
 
         self.running_data = self.get_data(indices)
 
-        _dir = self.dir + "/" + self.treated + "/hp/"
+        _dir = self.dir if _dir is None else _dir
+
         if not os.path.isdir(_dir):
             os.mkdir(_dir)
 
         if self._in_memory:
             new_data = hot_pixel_removal_3D(self.running_data, kernel).view(Data)
-            new_data.save(_dir + "/data_hp")
+            new_data.save(_dir + "/data.hdf5")
             urls = new_data.urls
         else:
             urls = self.running_data.apply_funcs([(hot_pixel_removal_2D, [kernel])],
-                                                 save=_dir + "/data_hp", text="Applying hot pixel removal",
+                                                 save=_dir + "/data.hdf5", text="Applying hot pixel removal",
                                                  operation=Operation.HP)
             if urls is None:
                 return
@@ -614,9 +647,10 @@ class Dataset():
             new_data = Data(urls.reshape(self.data.urls.shape), self.data.metadata,
                             self._in_memory)
 
-        return Dataset(_dir=_dir, data=new_data, dims=self.__dims, in_memory=self._in_memory)
+        return Dataset(_dir=_dir, data=new_data, dims=self.__dims, transformation=self.transformation,
+                       in_memory=self._in_memory)
 
-    def apply_threshold_removal(self, bottom=None, top=None, indices=None):
+    def apply_threshold_removal(self, bottom=None, top=None, indices=None, _dir=None):
         """
         Applies bottom threshold to Data, and saves the new data
         into disk.
@@ -634,17 +668,18 @@ class Dataset():
 
         self.running_data = self.get_data(indices)
 
-        _dir = self.dir + "/" + self.treated + "/threshold/"
+        _dir = self.dir if _dir is None else _dir
+
         if not os.path.isdir(_dir):
             os.mkdir(_dir)
 
         if self._in_memory:
             new_data = threshold_removal(self.running_data, bottom, top).view(Data)
-            new_data.save(_dir + "/data_threshold")
+            new_data.save(_dir + "/data.hdf5")
             urls = new_data.urls
         else:
             urls = self.running_data.apply_funcs([(threshold_removal, [bottom, top])],
-                                                 save=_dir + "/data_threshold", text="Applying threshold",
+                                                 save=_dir + "/data.hdf5", text="Applying threshold",
                                                  operation=Operation.THRESHOLD)
             if urls is None:
                 return
@@ -658,9 +693,10 @@ class Dataset():
             new_data = Data(urls.reshape(self.data.urls.shape), self.data.metadata,
                             self._in_memory)
 
-        return Dataset(_dir=_dir, data=new_data, dims=self.__dims, in_memory=self._in_memory)
+        return Dataset(_dir=_dir, data=new_data, dims=self.__dims, transformation=self.transformation,
+                       in_memory=self._in_memory)
 
-    def apply_roi(self, origin=None, size=None, center=None, indices=None):
+    def apply_roi(self, origin=None, size=None, center=None, indices=None, roi_dir=None):
         """
         Applies a region of interest to the data.
 
@@ -673,34 +709,43 @@ class Dataset():
         :param indices: Indices of the images to apply background subtraction.
             If None, the roi is applied to all the data.
         :type indices: Union[None, array_like]
+        :param roi_dir: Directory path for the new dataset
+        :type roi_dir: str
         :returns: dataset with data with roi applied.
             Note: To preserve consistence of shape between images, if `indices`
             is not None, only the data modified is returned.
         :rtype: Dataset
         """
 
-        roi_dir = self.dir + "/" + self.treated + "/roi/"
-        self.running_data = self.get_data(indices)
+        roi_dir = self.dir if roi_dir is None else roi_dir
         if not os.path.isdir(roi_dir):
             os.mkdir(roi_dir)
+
+        self.running_data = self.get_data(indices)
         if self._in_memory:
             new_data = apply_3D_ROI(self.running_data, origin, size, center).view(Data)
-            new_data.save(roi_dir + "/roi")
+            new_data.save(roi_dir + "/data.hdf5", new_shape=new_data.shape)
         else:
+            shape = numpy.append([self.running_data.shape[0]], apply_2D_ROI(self.running_data[0], origin, size, center).shape)
             urls = self.running_data.apply_funcs([(apply_2D_ROI, [origin, size, center])],
-                                                 save=roi_dir + "/data_roi", text="Applying roi",
-                                                 operation=Operation.ROI)
+                                                 save=roi_dir + "/data.hdf5", text="Applying roi",
+                                                 operation=Operation.ROI,
+                                                 new_shape=shape)
             if urls is None:
                 return
             new_data = Data(urls, self.running_data.metadata, self._in_memory)
+
+        transformation = ([apply_2D_ROI(axis, origin, size, center) for axis in self.transformation]
+                          if self.transformation else None)
         if indices is None:
             shape = list(self.data.shape)[:-2]
             shape.append(new_data.shape[-2])
             shape.append(new_data.shape[-1])
             new_data = new_data.reshape(shape)
-        return Dataset(_dir=roi_dir, data=new_data, dims=self.__dims, in_memory=self._in_memory)
+        return Dataset(_dir=roi_dir, data=new_data, dims=self.__dims, transformation=transformation,
+                       in_memory=self._in_memory)
 
-    def find_shift(self, dimension=None, h_max=0.5, h_step=0.01, indices=None):
+    def find_shift(self, dimension=None, steps=50, indices=None):
         """
         Find shift of the data or part of it.
 
@@ -714,9 +759,32 @@ class Dataset():
         :type indices: Union[None, array_like]
         :returns: Array with shift per frame.
         """
-        return shift_detection(self.get_data(indices, dimension), h_max, h_step)
+        return shift_detection(self.get_data(indices, dimension), steps)
 
-    def apply_shift(self, shift, dimension=None, shift_approach="fft", indices=None, callback=None):
+    def find_shift_along_dimension(self, dimension, steps=50, indices=None):
+        shift = []
+        for value in range(self.dims.get(dimension[0]).size):
+            s = self.find_shift([dimension[0], value], steps, indices)
+            shift.append(s)
+
+        return numpy.array(shift)
+
+    def apply_shift_along_dimension(self, shift, dimension, shift_approach="fft", indices=None,
+                                    callback=None, _dir=None):
+
+        dataset = self
+        for value in range(self.dims.get(dimension[0]).size):
+            data, rindices = self.get_data(indices=indices, dimension=[dimension[0], value],
+                                           return_indices=True)
+            frames = numpy.arange(self.get_data(indices=indices,
+                                  dimension=[dimension[0], value]).shape[0])
+            dataset = dataset.apply_shift(numpy.outer(shift[value], frames), [dimension[0], value],
+                                          shift_approach, indices, callback, _dir)
+
+        return dataset
+
+    def apply_shift(self, shift, dimension=None, shift_approach="fft", indices=None,
+                    callback=None, _dir=None):
         """
         Apply shift of the data or part of it and save new data into disk.
 
@@ -738,30 +806,41 @@ class Dataset():
         if not numpy.any(shift):
             return self
 
-        shift_dir = self.dir + "/" + self.treated + "/shift/"
-        if not os.path.isdir(shift_dir):
-            os.mkdir(shift_dir)
+        _dir = self.dir if _dir is None else _dir
 
-        data = self.get_data(indices, dimension)
+        if not os.path.isdir(_dir):
+            os.mkdir(_dir)
+
+        data, rindices = self.get_data(indices, dimension, return_indices=True)
         self._lock.acquire()
         self.operations_state[Operation.SHIFT] = 1
         self._lock.release()
+        _file = h5py.File(_dir + '/data.hdf5', 'a')
+        dataset_name = "dataset"
+        if "dataset" in _file:
+            _file.create_dataset("update_dataset", data=_file["dataset"])
+            dataset_name = "update_dataset"
+        else:
+            _file.create_dataset("dataset", self.get_data().shape, dtype=self.data.dtype)
 
         io_utils.advancement_display(0, len(data), "Applying shift")
         if dimension is not None:
+
             # Convert dimension and value into list
             if type(dimension[0]) is int:
                 dimension[0] = [dimension[0]]
                 dimension[1] = [dimension[1]]
             urls = []
-            for i in range(len(data)):
+            for i, idx in enumerate(rindices):
                 if not self.operations_state[Operation.SHIFT]:
+                    del _file["update_dataset"]
                     return
-                filename = shift_dir + "/data_shift" + str(i).zfill(4) + ".npy"
                 img = apply_shift(data[i], shift[:, i], shift_approach)
-                numpy.save(filename, img)
-                urls.append(DataUrl(file_path=filename, scheme='fabio'))
-                io_utils.advancement_display(i + 1, len(data), "Applying shift")
+                if shift[:, i].all() > 1:
+                    shift_approach = "linear"
+                _file[dataset_name][idx] = img
+                urls.append(DataUrl(file_path=_dir + '/data.hdf5', data_path="/dataset", data_slice=idx, scheme='silx'))
+                io_utils.advancement_display(i + 1, len(rindices), "Applying shift")
 
             # Replace specific urls that correspond to the modified data
             new_urls = numpy.array(self.data.urls, dtype=object)
@@ -787,11 +866,13 @@ class Dataset():
             urls = []
             for i in range(len(data)):
                 if not self.operations_state[Operation.SHIFT]:
+                    del _file["update_dataset"]
                     return
-                filename = shift_dir + "/data_shift" + str(i).zfill(4) + ".npy"
+                if shift[:, i].all() > 1:
+                    shift_approach = "linear"
                 img = apply_shift(data[i], shift[:, i], shift_approach)
-                numpy.save(filename, img)
-                urls.append(DataUrl(file_path=filename, scheme='fabio'))
+                _file[dataset_name][i] = img
+                urls.append(DataUrl(file_path=_dir + '/data.hdf5', data_path="/dataset", data_slice=i, scheme='silx'))
                 io_utils.advancement_display(i + 1, len(data), "Applying shift")
             if indices is not None:
                 new_urls = numpy.array(self.data.urls, dtype=object).flatten()
@@ -803,10 +884,17 @@ class Dataset():
         self.operations_state[Operation.SHIFT] = 0
         self._lock.release()
 
-        data = Data(new_urls.reshape(self.data.urls.shape), self.data.metadata, in_memory=self._in_memory)
-        return Dataset(_dir=shift_dir, data=data, dims=self.__dims, in_memory=self._in_memory)
+        if dataset_name == "update_dataset":
+            del _file["dataset"]
+            _file["dataset"] = _file["update_dataset"]
+            del _file["update_dataset"]
 
-    def find_and_apply_shift(self, dimension=None, h_max=0.5, h_step=0.01, shift_approach="fft",
+        _file.close()
+        data = Data(new_urls.reshape(self.data.urls.shape), self.data.metadata, in_memory=self._in_memory)
+        return Dataset(_dir=_dir, data=data, dims=self.__dims, transformation=self.transformation,
+                       in_memory=self._in_memory)
+
+    def find_and_apply_shift(self, dimension=None, steps=100, shift_approach="fft",
                              indices=None, callback=None):
         """
         Find the shift of the data or part of it and apply it.
@@ -823,7 +911,7 @@ class Dataset():
         :param Union[function, None] callback: Callback
         :returns: Dataset with the new data.
         """
-        shift = self.find_shift(dimension, h_max, h_step, indices=indices)
+        shift = self.find_shift(dimension, steps, indices=indices)
         return self.apply_shift(shift, dimension, indices=indices)
 
     def _cascade_nmf(self, num_components, iterations, vstep=None, hstep=None, indices=None):
@@ -909,7 +997,7 @@ class Dataset():
 
         :return: (H, W): The components matrix and the mixing matrix.
         """
-        bss_dir = self.dir + "/" + self.treated + "/bss/"
+        bss_dir = self.dir + "/bss"
         if not os.path.isdir(bss_dir):
             os.mkdir(bss_dir)
         if self._in_memory:
@@ -961,7 +1049,7 @@ class Dataset():
 
         :return: (H, W): The components matrix and the mixing matrix.
         """
-        bss_dir = self.dir + "/" + self.treated + "/bss/"
+        bss_dir = self.dir + "/bss/"
         if not os.path.isdir(bss_dir):
             os.mkdir(bss_dir)
 
@@ -1002,7 +1090,7 @@ class Dataset():
 
         :return: (H, W): The components matrix and the mixing matrix.
         """
-        bss_dir = self.dir + "/" + self.treated + "/bss/"
+        bss_dir = self.dir + "/bss/"
         if not os.path.isdir(bss_dir):
             os.mkdir(bss_dir)
 
@@ -1043,7 +1131,7 @@ class Dataset():
         return self.nmf(num_components, num_iter, error_step, cascade, H, W,
                         vstep, hstep, indices=indices)
 
-    def apply_moments(self, indices=None, chunk_shape=[100, 100]):
+    def apply_moments(self, indices=None, chunk_shape=[500, 500]):
         """
         Compute the COM, FWHM, skewness and kurtosis of the data for very dimension.
 
@@ -1055,11 +1143,12 @@ class Dataset():
         """
 
         self.running_data = self.get_data(indices)
+        data = self.get_data()
         if indices is None:
             indices = range(self.nframes)
         for axis, dim in self.dims:
             # Get motor values per image of the stack
-            values = numpy.array([self.running_data.metadata[i].get_value(kind=dim.kind, name=dim.name) for i in indices])
+            values = numpy.array([data.metadata[i].get_value(kind=dim.kind, name=dim.name) for i in indices])
             if self._in_memory:
                 # Data in memory
                 com, std, skew, kurt = compute_moments(values, self.running_data)
@@ -1095,7 +1184,8 @@ class Dataset():
 
         return self.moments_dims
 
-    def apply_fit(self, indices=None, dimension=None, int_thresh=None, chunk_shape=[100, 100]):
+    def apply_fit(self, indices=None, dimension=None, int_thresh=None,
+                  chunk_shape=[100, 100], _dir=None):
         """
         Fits the data around axis 0 and saves the new data into disk.
 
@@ -1114,7 +1204,10 @@ class Dataset():
             the new urls.
         :rtype: Dataset
         """
-        _dir = self.dir + "/" + self.treated + "/fit/"
+
+        _dir = self.dir if _dir is None else _dir
+        _dir += "/fit"
+
         if not os.path.isdir(_dir):
             os.mkdir(_dir)
 
@@ -1232,15 +1325,30 @@ class Dataset():
         else:
             new_urls = numpy.array(urls)
         data = Data(new_urls.reshape(self.data.urls.shape), self.data.metadata, in_memory=True)  # to modify
-        return Dataset(_dir=_dir, data=data, dims=self.__dims, in_memory=True)
+        return Dataset(_dir=_dir, data=data, dims=self.__dims, transformation=self.transformation, in_memory=True)
+
+    def compute_transformation(self, d, kind="magnification", rotate=False):
+
+        H, W = self.get_data(0).shape
+
+        if self.dims.ndim == 1 and kind == "rsm":
+            ffz = self.get_metadata_values(POSITIONER_METADATA, "ffz")[0]
+            mainx = -self.get_metadata_values(POSITIONER_METADATA, "mainx")[0]
+            self.transformation = compute_rsm(H, W, d, ffz, mainx)
+        else:
+            obx = self.get_metadata_values(POSITIONER_METADATA, "obx")[0]
+            obpitch = self.get_metadata_values(POSITIONER_METADATA, "obpitch")
+            obpitch = obpitch[len(obpitch) // 2]
+            mainx = -self.get_metadata_values(POSITIONER_METADATA, "mainx")[0]
+            self.transformation = compute_magnification(H, W, d, obx, obpitch, mainx)
 
     def __deepcopy__(self, memo):
         """
         Create copy of the dataset. The data numpy array is also copied using
         deep copy. The rest of the attributes are the same.
         """
-        dataset = type(self)(self.dir, data=self.data, filenames=self.filenames,
-                             first_filename=self.first_filename, treated=self.treated)
+        dataset = type(self)(self.dir, data=self.data, dims=self.__dims,
+                             in_memory=self.in_memory, copy_files=True)
         dataset.dims = copy.deepcopy(self.__dims, memo)
         return dataset
 
@@ -1260,18 +1368,16 @@ class Data(numpy.ndarray):
     def __new__(cls, urls, metadata, in_memory=True, data=None):
         urls = numpy.asarray(urls)
         if in_memory:
-            if data is not None and urls.shape == data.shape[:-2]:
-                input_data = data
-            else:
+            if data is None or urls.shape != data.shape[:-2]:
                 # Create array as stack of images
                 input_data = []
                 for url in urls.flatten():
-                    input_data += [utils.get_data(url)]
-                input_data = numpy.asarray(input_data)
+                    input_data.append(utils.get_data(url))
+                data = numpy.asarray(input_data)
             shape = list(urls.shape)
-            shape.append(input_data.shape[-2])
-            shape.append(input_data.shape[-1])
-            obj = input_data.reshape(shape).view(cls)
+            shape.append(data.shape[-2])
+            shape.append(data.shape[-1])
+            obj = data.reshape(shape).view(cls)
         else:
             # Access image one at a time using url
             obj = super(Data, cls).__new__(cls, urls.shape)
@@ -1302,7 +1408,7 @@ class Data(numpy.ndarray):
             if len(data.shape) < 3:
                 data = data.view(numpy.ndarray)
             else:
-                if isinstance(indices[0], slice):
+                if isinstance(indices, tuple):
                     data.urls = self.urls[indices[0]]
                     data.metadata = self.metadata[indices[0]]
                 else:
@@ -1369,7 +1475,7 @@ class Data(numpy.ndarray):
         else:
             return super(Data, self).ndim + 2
 
-    def apply_funcs(self, funcs=[], indices=None, save=False, text="", operation=None):
+    def apply_funcs(self, funcs=[], indices=None, save=False, text="", operation=None, new_shape=None):
         """
         Method that applies a series of functions into the data. It can save the images
         into disk or return them.
@@ -1401,30 +1507,57 @@ class Data(numpy.ndarray):
         self._lock.acquire()
         self.operations[operation] = 1
         self._lock.release()
+        _file = h5py.File(save, 'a')
+        dataset_name = "dataset"
+        new_shape = self.shape if new_shape is None else tuple(new_shape)
+
+        if "dataset" in _file:
+            if new_shape != _file["dataset"].shape:
+                _file.create_dataset("update_dataset", new_shape, dtype=self.dtype)
+            else:
+                _file.create_dataset("update_dataset", shape=_file["dataset"].shape, dtype=_file["dataset"].dtype)
+                for i, img in enumerate(_file["dataset"]):
+                    _file["update_dataset"][i] = img
+            dataset_name = "update_dataset"
+        else:
+            new_shape = self.shape if new_shape is None else new_shape
+            _file.create_dataset("dataset", new_shape, dtype=self.dtype)
+
         for i in indices:
             if operation is not None and not self.operations[operation]:
-                if save:
-                    for j in indices:
-                        if j != i:
-                            filename = save + str(j).zfill(4) + ".npy"
-                            os.remove(filename)
-                        else:
-                            break
+                del _file["update_dataset"]
                 return
+            #     if save:
+            #         for j in indices:
+            #             if j != i:
+            #                 filename = save + str(j).zfill(4) + ".npy"
+            #                 os.remove(filename)
+            #             else:
+            #                 break
+            #     return
             img = self[int(i)]
             for f, args in funcs:
                 img = f(*([img] + args))
             if save:
-                filename = save + str(i).zfill(4) + ".npy"
-                numpy.save(filename, img)
-                urls.append(DataUrl(file_path=filename, scheme='fabio'))
+                _file[dataset_name][i] = img
+                urls.append(DataUrl(file_path=save, data_path="/dataset", data_slice=i, scheme='silx'))
+                # filename = save + str(i).zfill(4) + ".npy"
+                # numpy.save(filename, img)
+                # urls.append(DataUrl(file_path=filename, scheme='fabio'))
             io_utils.advancement_display(i + 1, len(self.urls.flatten()), text)
         self._lock.acquire()
         self.operations[operation] = 0
         self._lock.release()
+
+        if dataset_name == "update_dataset":
+            del _file["dataset"]
+            _file["dataset"] = _file["update_dataset"]
+            del _file["update_dataset"]
+
+        _file.close()
         return numpy.array(urls)
 
-    def save(self, path, indices=None):
+    def save(self, path, indices=None, new_shape=None):
         """
         Save the data into `path` folder and replace Data urls.
         TODO: check if urls already exist and, if so, modify only urls[indices].
@@ -1440,10 +1573,24 @@ class Data(numpy.ndarray):
             data = self
         else:
             data = self[indices]
+
+        _file = h5py.File(path, 'a')
+
+        new_shape = self.shape if new_shape is None else tuple(new_shape)
+
+        if "dataset" not in _file:
+            _file.create_dataset("dataset", self.shape, dtype=self.dtype)
+        elif new_shape != _file["dataset"].shape:
+            del _file["dataset"]
+            _file.create_dataset("dataset", new_shape, dtype=self.dtype)
+
         for i, img in enumerate(data):
-            filename = path + str(i).zfill(4) + ".npy"
-            numpy.save(filename, img)
-            urls.append(DataUrl(file_path=filename, scheme='fabio'))
+            _file["dataset"][i] = img
+            urls.append(DataUrl(file_path=path, data_path="/dataset", data_slice=i, scheme='silx'))
+        #     filename = path + str(i).zfill(4) + ".npy"
+        #     numpy.save(filename, img)
+        #     urls.append(DataUrl(file_path=filename, scheme='fabio'))
+        _file.close()
         self.urls = numpy.asarray(urls)
 
     def convert_to_hdf5(self, _dir):
@@ -1514,7 +1661,9 @@ class Data(numpy.ndarray):
                 raise TypeError("Axis must be None, 0 or 1")
         else:
             if axis == 0:
-                if not data.shape[0]:
+                if data.size == 0:
+                    return numpy.array([])
+                elif not data.shape[0]:
                     return numpy.zeros(data[0].shape)
                 zsum = numpy.array(data[0], dtype=numpy.float64)
                 for i in range(1, len(data)):
