@@ -26,7 +26,7 @@
 
 __authors__ = ["J. Garriga"]
 __license__ = "MIT"
-__date__ = "11/05/2020"
+__date__ = "09/12/2020"
 
 import copy
 import unittest
@@ -37,7 +37,7 @@ import shutil
 from silx.io.url import DataUrl
 
 from darfix.test import utils
-from darfix.core.dataset import Data, Dimension, POSITIONER_METADATA
+from darfix.core.dataset import Data
 
 
 class TestDataset(unittest.TestCase):
@@ -161,7 +161,7 @@ class TestDataset(unittest.TestCase):
         dark_dataset = utils.createDataset(dark, in_memory=True)
 
         new_dataset = dataset.apply_background_subtraction(background=dark_dataset, method="mean")
-        new_dataset.in_memory(True)
+        new_dataset.in_memory = True
         numpy.testing.assert_array_equal(expected, new_dataset.data)
 
         # Background data in disk
@@ -169,7 +169,7 @@ class TestDataset(unittest.TestCase):
         dark_dataset = utils.createDataset(dark, in_memory=False)
 
         new_dataset = dataset.apply_background_subtraction(background=dark_dataset, method="mean")
-        new_dataset.in_memory(True)
+        new_dataset.in_memory = True
         numpy.testing.assert_array_equal(expected, new_dataset.data)
 
     def test_bs_median_in_disk(self):
@@ -211,7 +211,7 @@ class TestDataset(unittest.TestCase):
         dark_dataset = utils.createDataset(dark, in_memory=True)
 
         new_dataset = dataset.apply_background_subtraction(background=dark_dataset, method="median")
-        new_dataset.in_memory(True)
+        new_dataset.in_memory = True
         numpy.testing.assert_array_equal(expected, new_dataset.data)
 
         # Background data in disk
@@ -219,7 +219,19 @@ class TestDataset(unittest.TestCase):
         dark_dataset = utils.createDataset(dark, in_memory=False)
 
         new_dataset = dataset.apply_background_subtraction(background=dark_dataset, method="median")
-        new_dataset.in_memory(True)
+        new_dataset.in_memory = True
+        numpy.testing.assert_array_equal(expected, new_dataset.data)
+
+        # Compute background with chunks
+        new_dataset = dataset.apply_background_subtraction(background=dark_dataset, method="median",
+                                                           chunk_shape=[2, 2])
+        new_dataset.in_memory = True
+        numpy.testing.assert_array_equal(expected, new_dataset.data)
+
+        # Compute background with step
+        new_dataset = dataset.apply_background_subtraction(background=dark_dataset, method="median",
+                                                           step=2)
+        new_dataset.in_memory = True
         numpy.testing.assert_array_equal(expected, new_dataset.data)
 
     def test_hp_in_memory(self):
@@ -243,6 +255,27 @@ class TestDataset(unittest.TestCase):
         self.assertNotEqual(hp_dataset.data.urls[0], dataset.data.urls[0])
         numpy.testing.assert_equal(dataset.data[5], hp_dataset.data[5])
 
+    def test_threshold_in_memory(self):
+        """ Tests the threshold removal function with data in memory"""
+        indices = [0, 1]
+        threshold_dataset = self.dataset.apply_threshold_removal(indices=indices)
+
+        self.assertEqual(threshold_dataset.nframes, self.dataset.nframes)
+        self.assertNotEqual(threshold_dataset.data.urls[0], self.dataset.data.urls[0])
+        numpy.testing.assert_equal(self.dataset.data.metadata, threshold_dataset.data.metadata)
+        numpy.testing.assert_equal(self.dataset.data[2], threshold_dataset.data[2])
+
+    def test_threshold_in_disk(self):
+        """ Tests the threshold removal function with data in disk"""
+
+        dataset = utils.createRandomDataset(dims=(100, 100), nb_data_files=10, in_memory=False)
+        indices = [0, 1, 2, 3, 4]
+        threshold_dataset = dataset.apply_threshold_removal(indices=indices)
+
+        self.assertEqual(threshold_dataset.nframes, dataset.nframes)
+        self.assertNotEqual(threshold_dataset.data.urls[0], dataset.data.urls[0])
+        numpy.testing.assert_equal(dataset.data[5], threshold_dataset.data[5])
+
     def test_roi_in_memory(self):
         """ Tests the roi function with data in memory"""
         new_dataset = self.dataset.apply_roi(origin=[0, 0], size=[20, 20])
@@ -272,7 +305,7 @@ class TestDataset(unittest.TestCase):
     def test_apply_shift(self):
         """ Tests the shift correction"""
 
-        new_dataset = self.dataset.find_and_apply_shift()
+        new_dataset = self.dataset.apply_shift(shift=numpy.array([[0, 0.5, 1], [0, 0.5, 1]]))
 
         self.assertEqual(new_dataset.nframes, self.dataset.nframes)
         self.assertNotEqual(new_dataset.data.urls[0], self.dataset.data.urls[0])
@@ -377,6 +410,24 @@ class TestDataset(unittest.TestCase):
         self.assertEqual(H.shape, (n_components, len(dataset.data[0].flatten())))
         self.assertEqual(W.shape, (len(indices), n_components))
 
+    def test_apply_fit_in_memory(self):
+        """ Tests fitting data in memory"""
+        new_dataset = self.dataset.apply_fit()
+
+        self.assertEqual(new_dataset.nframes, self.dataset.nframes)
+        self.assertNotEqual(new_dataset.data.urls[0], self.dataset.data.urls[0])
+
+    def test_apply_fit_in_disk(self):
+        """Tests fitting data in disk"""
+
+        dataset = utils.createRandomDataset(dims=(100, 100), nb_data_files=10, in_memory=False)
+        indices = [0, 1, 2, 3, 4]
+        fit_dataset = dataset.apply_fit(indices=indices)
+
+        self.assertEqual(fit_dataset.nframes, dataset.nframes)
+        self.assertNotEqual(fit_dataset.data.urls[0], dataset.data.urls[0])
+        numpy.testing.assert_equal(fit_dataset.data[5], dataset.data[5])
+
 
 class TestData(unittest.TestCase):
 
@@ -470,7 +521,7 @@ class TestData(unittest.TestCase):
     def test_save(self):
         data = numpy.random.rand(10, 100, 100).view(Data)
         # data.in_memory = True
-        data.save(self._dir)
+        data.save(self._dir + "/data.hdf5")
 
         numpy.testing.assert_array_equal(data.shape, (10, 100, 100))
 
@@ -521,150 +572,5 @@ class TestData(unittest.TestCase):
         shutil.rmtree(self._dir)
 
 
-class TestDimensions(unittest.TestCase):
-
-    """Tests for class Dimension in `dataset.py`"""
-
-    def setUp(self):
-        """"
-        Creating random dataset with specific headers.
-        """
-        self._dir = tempfile.mkdtemp()
-        counter_mne = "a b c d e f g h"
-        motor_mne = "x y z k h m n"
-        dims = (10, 100, 100)
-        # Create headers
-        header = []
-        # Dimensions for reshaping
-        a = numpy.random.rand(2)
-        b = numpy.random.rand(5)
-        motors = numpy.random.rand(7)
-        for i in numpy.arange(10):
-            header.append({})
-            header[i]["HeaderID"] = i
-            header[i]["counter_mne"] = counter_mne
-            header[i]["motor_mne"] = motor_mne
-            header[i]["counter_pos"] = ""
-            header[i]["motor_pos"] = ""
-            for c in counter_mne:
-                header[i]["counter_pos"] += str(numpy.random.rand(1)[0]) + " "
-            for j, m in enumerate(motor_mne.split()):
-                if m == "z":
-                    header[i]["motor_pos"] += str(a[i % 2]) + " "
-                elif m == "m":
-                    header[i]["motor_pos"] += str(b[i % 5]) + " "
-                else:
-                    header[i]["motor_pos"] += str(motors[j]) + " "
-
-        data = numpy.zeros(dims)
-        background = numpy.random.random(dims)
-        idxs = [0, 2, 4]
-        data[idxs] += background[idxs]
-        self.dataset = utils.createDataset(data=data, header=header, _dir=self._dir)
-
-    def test_add_one_dimension(self):
-        """ Tests the correct add of a dimension """
-
-        dimension = Dimension(POSITIONER_METADATA, "test", 20)
-        self.dataset.add_dim(0, dimension)
-        saved_dimension = self.dataset.dims.get(0)
-        self.assertEqual(saved_dimension.name, "test")
-        self.assertEqual(saved_dimension.kind, POSITIONER_METADATA)
-        self.assertEqual(saved_dimension.size, 20)
-
-    def test_add_several_dimensions(self):
-        """ Tests the correct add of several dimensions """
-
-        dimension1 = Dimension(POSITIONER_METADATA, "test1", 20)
-        dimension2 = Dimension(POSITIONER_METADATA, "test2", 30)
-        dimension3 = Dimension(POSITIONER_METADATA, "test3", 40)
-
-        self.dataset.add_dim(0, dimension1)
-        self.dataset.add_dim(1, dimension2)
-        self.dataset.add_dim(2, dimension3)
-        self.assertEqual(self.dataset.dims.ndim, 3)
-
-    def test_remove_dimension(self):
-        """ Tests the correct removal of a dimension """
-
-        dimension = Dimension(POSITIONER_METADATA, "test", 20)
-        self.dataset.add_dim(0, dimension)
-        self.dataset.remove_dim(0)
-
-        self.assertEqual(self.dataset.dims.ndim, 0)
-
-    def test_remove_dimensions(self):
-        """ Tests the correct removal of several dimensions """
-
-        dimension1 = Dimension(POSITIONER_METADATA, "test1", 20)
-        dimension2 = Dimension(POSITIONER_METADATA, "test2", 30)
-        dimension3 = Dimension(POSITIONER_METADATA, "test3", 40)
-
-        self.dataset.add_dim(0, dimension1)
-        self.dataset.add_dim(1, dimension2)
-        self.dataset.add_dim(2, dimension3)
-
-        self.dataset.remove_dim(0)
-        self.dataset.remove_dim(2)
-
-        self.assertEqual(self.dataset.dims.ndim, 1)
-        self.assertEqual(self.dataset.dims.get(1).name, "test2")
-
-    def test_find_dimensions(self):
-        """ Tests the correct finding of the dimensions"""
-
-        self.dataset.find_dimensions(POSITIONER_METADATA)
-        self.assertEqual(self.dataset.dims.ndim, 2)
-        self.assertEqual(self.dataset.dims.get(0).name, "z")
-        self.assertEqual(self.dataset.dims.get(1).name, "m")
-
-    def test_reshaped_data(self):
-        """ Tests the correct reshaping of the data """
-        self.dataset.find_dimensions(POSITIONER_METADATA)
-
-        dataset = self.dataset.reshape_data()
-        self.assertEqual(dataset.data.shape, (2, 5, 100, 100))
-
-    def test_find_shift(self):
-        """ Tests the shift detection with dimensions and indices"""
-        self.dataset.find_dimensions(POSITIONER_METADATA)
-
-        dataset = self.dataset.reshape_data()
-        indices = [1, 2, 3, 4]
-        shift = dataset.find_shift(dimension=[0, 1], indices=indices)
-
-        self.assertEqual(len(shift), 0)
-
-        shift = dataset.find_shift(dimension=[1, 1], indices=indices)
-
-        self.assertEqual(shift.shape, (2, 1))
-
-    def test_apply_shift(self):
-        """ Tests the shift correction with dimensions and indices"""
-        self.dataset.find_dimensions(POSITIONER_METADATA)
-        dataset = self.dataset.reshape_data()
-        new_dataset = dataset.find_and_apply_shift(dimension=[1, 1], indices=[1, 2, 3, 4])
-
-        self.assertEqual(new_dataset.data.urls[0, 0], dataset.data.urls[0, 0])
-        self.assertNotEqual(new_dataset.data.urls[0, 1], dataset.data.urls[0, 1])
-
-    def test_data_reshaped_data(self):
-        """ Tests that data and reshaped data have same values """
-        self.dataset.find_dimensions(POSITIONER_METADATA)
-        dataset = self.dataset.reshape_data()
-
-        numpy.testing.assert_array_equal(dataset.data[0, 0], self.dataset.data[0])
-
-    def test_clear_dimensions(self):
-        """ Tests the clear dimensions function """
-        self.dataset.find_dimensions(POSITIONER_METADATA)
-        self.dataset.clear_dims()
-
-        self.assertEqual(self.dataset.dims.ndim, 0)
-
-    def tearDown(self):
-        shutil.rmtree(self._dir)
-
-
 if __name__ == '__main__':
     unittest.main()
