@@ -26,7 +26,7 @@
 
 __authors__ = ["J. Garriga"]
 __license__ = "MIT"
-__date__ = "13/09/2019"
+__date__ = "16/06/2021"
 
 
 import unittest
@@ -41,7 +41,7 @@ except ImportError:
 
 from darfix.core import imageRegistration
 from darfix.test import utils
-from darfix.core.dataset import POSITIONER_METADATA
+from darfix.core.dimension import POSITIONER_METADATA
 
 
 class TestImageRegistration(unittest.TestCase):
@@ -88,14 +88,14 @@ class TestImageRegistration(unittest.TestCase):
 
     def test_improve_shift(self):
         """ Tests the shift improvement"""
-        h = imageRegistration.improve_linear_shift(self.data, [1, 1], 0.1, 0.1, shift_approach='fft')
-        numpy.testing.assert_allclose(h, [0, 0])
-        h = imageRegistration.improve_linear_shift(self.data, [1, 1], 0.1, 0.1, shift_approach='linear')
-        numpy.testing.assert_allclose([0, 0], [0, 0])
+        h = imageRegistration.improve_linear_shift(self.data, [1, 1], 0.1, 0.1, 1, shift_approach='fft')
+        numpy.testing.assert_allclose(h, 0.)
+        h = imageRegistration.improve_linear_shift(self.data, [1, 1], 0.1, 0.1, 1, shift_approach='linear')
+        numpy.testing.assert_allclose(h, 0.)
 
     @unittest.skipUnless(scipy, "scipy is missing")
     def test_shift_detection10(self):
-        """ Tests the shift detection with tolerance of 5 decimals"""
+        """ Tests the shift detection with tolerance of 3 decimals"""
         first_frame = numpy.zeros((100, 100))
         # Simulating a series of frame with information in the middle.
         first_frame[25:75, 25:75] = numpy.random.randint(50, 300, size=(50, 50))
@@ -104,12 +104,12 @@ class TestImageRegistration(unittest.TestCase):
         for i in range(9):
             data += [numpy.fft.ifftn(scipy.ndimage.fourier_shift(numpy.fft.fftn(data[-1]), shift)).real]
         data = numpy.asanyarray(data, dtype=numpy.int16)
-        optimal_shift = imageRegistration.shift_detection(data, 2)
+        optimal_shift = imageRegistration.shift_detection(data, 100, shift_approach="fft")
 
         shift = [[0, -1, -2, -3, -4, -5, -6, -7, -8, -9],
                  [0, 0, 0, 0, 0, 0, 0, 0, 0, 0]]
 
-        numpy.testing.assert_allclose(shift, optimal_shift, rtol=1e-05)
+        numpy.testing.assert_allclose(shift, optimal_shift, rtol=1e-03)
 
     @unittest.skipUnless(scipy, "scipy is missing")
     def test_shift_detection01(self):
@@ -122,12 +122,12 @@ class TestImageRegistration(unittest.TestCase):
         for i in range(9):
             data += [numpy.fft.ifftn(scipy.ndimage.fourier_shift(numpy.fft.fftn(data[-1]), shift)).real]
         data = numpy.asanyarray(data, dtype=numpy.int16)
-        optimal_shift = imageRegistration.shift_detection(data, 2)
+        optimal_shift = imageRegistration.shift_detection(data, 100, shift_approach="fft")
 
         shift = [[0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
                  [0, -1, -2, -3, -4, -5, -6, -7, -8, -9]]
 
-        numpy.testing.assert_allclose(shift, optimal_shift, rtol=1e-05)
+        numpy.testing.assert_allclose(shift, optimal_shift, rtol=1e-03)
 
     @unittest.skipUnless(scipy, "scipy is missing")
     def test_shift_detection11(self):
@@ -141,7 +141,7 @@ class TestImageRegistration(unittest.TestCase):
             data += [numpy.fft.ifftn(scipy.ndimage.fourier_shift(numpy.fft.fftn(data[-1]), shift)).real]
         data = numpy.asanyarray(data, dtype=numpy.int16)
 
-        optimal_shift = imageRegistration.shift_detection(data, 2)
+        optimal_shift = imageRegistration.shift_detection(data, 100)
 
         shift = [[0, -1, -2, -3, -4, -5, -6, -7, -8, -9],
                  [0, -1, -2, -3, -4, -5, -6, -7, -8, -9]]
@@ -159,8 +159,7 @@ class TestImageRegistration(unittest.TestCase):
         for i in range(9):
             data += [numpy.fft.ifftn(scipy.ndimage.fourier_shift(numpy.fft.fftn(data[-1]), shift)).real]
         data = numpy.asanyarray(data, dtype=numpy.int16)
-        optimal_shift = imageRegistration.shift_detection(data, 1)
-
+        optimal_shift = imageRegistration.shift_detection(data, 100)
         shift = [[0, -0.5, -1, -1.5, -2, -2.5, -3, -3.5, -4, -4.5],
                  [0, -0.2, -0.4, -0.6, -0.8, -1, -1.2, -1.4, -1.6, -1.8]]
 
@@ -170,7 +169,7 @@ class TestImageRegistration(unittest.TestCase):
         """ Tests the shift correction of a [0,0] shift."""
 
         data = imageRegistration.shift_correction(self.data, numpy.outer([0, 0], numpy.arange(3)))
-        numpy.testing.assert_allclose(data, self.data, rtol=1e-05)
+        numpy.testing.assert_allclose(data, self.data, rtol=1e-03)
 
     def test_shift_correction01(self):
         """ Tests the shift correction of a [0,1] shift."""
@@ -192,7 +191,7 @@ class TestImageRegistration(unittest.TestCase):
                                  [1, 2, 3, 4, 5]]])
 
         data = imageRegistration.shift_correction(self.data, numpy.outer([1, 0], numpy.arange(3)))
-        numpy.testing.assert_allclose(data, expected, rtol=1e-05)
+        numpy.testing.assert_allclose(data, expected, rtol=1e-03)
 
     def test_shift_correction10(self):
         """ Tests the shift correction of a [1,0] shift."""
@@ -288,7 +287,7 @@ class TestReshapedShift(unittest.TestCase):
                 if m == "z":
                     header[i]["motor_pos"] += str(self.first_dim[i % 5]) + " "
                 elif m == "m":
-                    header[i]["motor_pos"] += str(self.second_dim[i % 2]) + " "
+                    header[i]["motor_pos"] += str(self.second_dim[int(i > 4)]) + " "
                 else:
                     header[i]["motor_pos"] += str(motors[j]) + " "
 
@@ -302,7 +301,7 @@ class TestReshapedShift(unittest.TestCase):
         data = [self.first_frame]
         shift = [0.5, 0.2]
         for i in range(1, 10):
-            if i % 2:
+            if i < 5:
                 data += [numpy.fft.ifftn(scipy.ndimage.fourier_shift(numpy.fft.fftn(data[-1]), shift)).real]
             else:
                 data += [data[-1]]
@@ -313,7 +312,7 @@ class TestReshapedShift(unittest.TestCase):
         dataset = self.dataset.reshape_data()
 
         # Detects shift using only images where value 1 of dimension 1 is fixed
-        optimal_shift = dataset.find_shift(dimension=[1, 1], h_max=1)
+        optimal_shift = dataset.find_shift(dimension=[1, 0])
 
         shift = [[0, -0.5, -1, -1.5, -2],
                  [0, -0.2, -0.4, -0.6, -0.8]]
@@ -334,10 +333,10 @@ class TestReshapedShift(unittest.TestCase):
         dataset = self.dataset.reshape_data()
 
         # Detects shift using only images where value 1 of dimension 1 is fixed
-        optimal_shift = dataset.find_shift(dimension=[1, 0], h_max=1)
+        optimal_shift = dataset.find_shift(dimension=[0, 0])
 
-        shift = [[0, -1, -2, -3, -4],
-                 [0, -0.4, -0.8, -1.2, -1.6]]
+        shift = [[0, -2.5],
+                 [0, -1]]
 
         numpy.testing.assert_allclose(shift, optimal_shift, rtol=1e-01)
 
@@ -347,7 +346,7 @@ class TestReshapedShift(unittest.TestCase):
         data = [self.first_frame]
         shift = [0.5, 0.2]
         for i in range(1, 10):
-            if i % 2:
+            if i < 5:
                 data += [numpy.fft.ifftn(scipy.ndimage.fourier_shift(numpy.fft.fftn(data[-1]), shift)).real]
             else:
                 data += [data[-1]]
@@ -358,11 +357,11 @@ class TestReshapedShift(unittest.TestCase):
         self.dataset.find_dimensions(POSITIONER_METADATA)
         dataset = self.dataset.reshape_data()
 
-        dataset = dataset.find_and_apply_shift(dimension=[1, 1], h_max=1)
+        dataset = dataset.find_and_apply_shift(dimension=[1, 0])
 
-        for frame in dataset.data.take(1, 1):
+        for frame in dataset.data.take(0, 0):
             # Check if the difference between the shifted frames and the sample frame is small enough
-            self.assertTrue((abs(data[1] - frame) < 5).all())
+            self.assertTrue((abs(data[0] - frame) < 6).all())
 
     def tearDown(self):
         shutil.rmtree(self._dir)
