@@ -26,7 +26,7 @@
 
 __authors__ = ["J. Garriga"]
 __license__ = "MIT"
-__date__ = "16/04/2020"
+__date__ = "19/05/2021"
 
 import logging
 import os
@@ -67,6 +67,10 @@ class DatasetSelectionWidget(qt.QTabWidget):
         self._darkFilenameData = FilenameSelectionWidget(parent=self)
         self.addTab(self._darkFilenameData, 'dark data')
 
+        # Treated data
+        self._treatedDirData = DirSelectionWidget(parent=self)
+        self.addTab(self._treatedDirData, 'treated data')
+
         self._dataset = None
         self.bg_dataset = None
         self.indices = None
@@ -78,6 +82,8 @@ class DatasetSelectionWidget(qt.QTabWidget):
         self.setRawFilenames = self._rawFilesData.setFiles
         self.setRawFilename = self._rawFilenameData.setFilename
         self.setDarkFilename = self._darkFilenameData.setFilename
+        self.getTreatedDir = self._treatedDirData.getDir
+        self.setTreatedDir = self._treatedDirData.setDir
 
         self._inDiskCB.stateChanged.connect(self.__inDisk)
 
@@ -85,19 +91,33 @@ class DatasetSelectionWidget(qt.QTabWidget):
         """
         Loads the dataset from the filenames.
         """
-        if not self._rawFilesData.getFiles():
-            _dir = os.path.dirname(self._rawFilenameData.getFilename())
-            self._dataset = Dataset(_dir=_dir,
-                                    in_memory=not self._inDisk)
-        else:
-            self._dataset = Dataset(_dir=self._rawFilesData.getDir(),
-                                    filenames=self._rawFilesData.getFiles(),
-                                    in_memory=not self._inDisk)
+        try:
+            _dir = self._treatedDirData.getDir()
+            if not self._rawFilesData.getFiles():
+                _dir = _dir if _dir != "" else os.path.dirname(self._rawFilenameData.getFilename())
+                self._dataset = Dataset(_dir=_dir,
+                                        first_filename=self._rawFilenameData.getFilename(),
+                                        in_memory=not self._inDisk, copy_files=True)
+            else:
+                _dir = _dir if _dir != "" else self._rawFilesData.getDir()
+                self._dataset = Dataset(_dir=_dir,
+                                        filenames=self._rawFilesData.getFiles(),
+                                        in_memory=not self._inDisk, copy_files=True)
 
-        dark_filename = self._darkFilenameData.getFilename()
-        if dark_filename != "":
-            self.bg_dataset = Dataset(_dir=os.path.dirname(dark_filename))
-        # self._dataset.sigProgressChanged.connect(self.updateProgress)
+            dark_filename = self._darkFilenameData.getFilename()
+            if dark_filename == "":
+                self.bg_dataset = None
+            elif _dir != "":
+                if not os.path.isdir(_dir + "/dark"):
+                    os.mkdir(_dir + "/dark")
+                self.bg_dataset = Dataset(_dir=_dir + "/dark",
+                                          first_filename=dark_filename, copy_files=True)
+            else:
+                self.bg_dataset = Dataset(_dir=os.path.dirname(dark_filename), copy_files=True)
+            return True
+        except Exception as e:
+            raise e
+            return False
 
     @property
     def dataset(self):
@@ -200,6 +220,48 @@ class FilesSelectionWidget(qt.QWidget):
         return None
 
 
+class DirSelectionWidget(qt.QWidget):
+    """
+    Widget used to obtain a filename (manually or from a file)
+    """
+
+    dirChanged = qt.Signal()
+
+    def __init__(self, parent=None):
+        qt.QWidget.__init__(self, parent)
+
+        self._dir = qt.QLineEdit('', parent=self)
+        self._dir.editingFinished.connect(self.dirChanged)
+        self._addButton = qt.QPushButton("Upload directory", parent=self)
+        # self._okButton =  qt.QPushButton("Ok", parent=self)
+        self._addButton.pressed.connect(self._uploadDir)
+        # self._okButton.pressed.connect(self.close)
+        self.setLayout(qt.QHBoxLayout())
+
+        self.layout().addWidget(self._dir)
+        self.layout().addWidget(self._addButton)
+        # self.layout().addWidget(self._okButton)
+
+    def _uploadDir(self):
+        """
+        Loads the file from a FileDialog.
+        """
+        fileDialog = qt.QFileDialog()
+        fileDialog.setOption(qt.QFileDialog.ShowDirsOnly)
+        fileDialog.setFileMode(qt.QFileDialog.Directory)
+        if fileDialog.exec_():
+            self._dir.setText(fileDialog.directory().absolutePath())
+            self.dirChanged.emit()
+        else:
+            _logger.warning("Could not open directory")
+
+    def getDir(self):
+        return str(self._dir.text())
+
+    def setDir(self, _dir):
+        self._dir.setText(str(_dir))
+
+
 class FilenameSelectionWidget(qt.QWidget):
     """
     Widget used to obtain a filename (manually or from a file)
