@@ -26,7 +26,7 @@
 
 __authors__ = ["J. Garriga"]
 __license__ = "MIT"
-__date__ = "11/12/2020"
+__date__ = "20/07/2021"
 
 import logging
 
@@ -219,10 +219,13 @@ class NoiseRemovalWidget(qt.QMainWindow):
         self._sv.setColormap(Colormap(name=darfix.config.DEFAULT_COLORMAP_NAME,
                                       normalization=darfix.config.DEFAULT_COLORMAP_NORM))
 
-        self._size = self._parametersDock.hpSizeCB.currentText()
-        self._method = self._parametersDock.bsMethodsCB.currentText()
+        self._size = None
+        self._method = None
         self._background = self._parametersDock.bsBackgroundCB.currentText()
         self._bottom_threshold = self._parametersDock.bottomLE.text()
+        self._step = self._parametersDock.step.text()
+        self._chunks = [int(self._parametersDock.verticalChunkSize.text()),
+                        int(self._parametersDock.horizontalChunkSize.text())]
         self.setCentralWidget(self._sv)
         self.addDockWidget(qt.Qt.BottomDockWidgetArea, self._parametersDock)
 
@@ -277,20 +280,21 @@ class NoiseRemovalWidget(qt.QMainWindow):
         else:
             bg = None
 
-        chunks = None
-        step = None
         if self._parametersDock.inDiskWidget.isVisible():
             if self._parametersDock.inDiskCheckbox.isChecked():
-                chunks = [int(self._parametersDock.verticalChunkSize.text()),
-                          int(self._parametersDock.horizontalChunkSize.text())]
+                self._step = None
+                self._chunks = [int(self._parametersDock.verticalChunkSize.text()),
+                                int(self._parametersDock.horizontalChunkSize.text())]
             else:
-                step = int(self._parametersDock.step.text())
+                self._chunks = None
+                self._step = int(self._parametersDock.step.text())
 
         self._thread = OperationThread(self, self._update_dataset.apply_background_subtraction)
         self._method = self._parametersDock.bsMethodsCB.currentText()
-        self._thread.setArgs(bg, self._method, self.indices, step, chunks)
+        self._thread.setArgs(bg, self._method, self.indices, self._step, self._chunks)
         self._thread.finished.connect(self._updateData)
-        self._parametersDock.abortBS.show()
+        if not self._update_dataset.in_memory:
+            self._parametersDock.abortBS.show()
         self._thread.start()
         self._setComputingState(True)
 
@@ -303,7 +307,8 @@ class NoiseRemovalWidget(qt.QMainWindow):
         self._thread = OperationThread(self, self._update_dataset.apply_hot_pixel_removal)
         self._thread.setArgs(int(self._size), self.indices)
         self._thread.finished.connect(self._updateData)
-        self._parametersDock.abortHP.show()
+        if not self._update_dataset.in_memory:
+            self._parametersDock.abortHP.show()
         self._thread.start()
         self._setComputingState(True)
 
@@ -316,7 +321,8 @@ class NoiseRemovalWidget(qt.QMainWindow):
         self._thread = OperationThread(self, self._update_dataset.apply_threshold_removal)
         self._thread.setArgs(int(self._bottom_threshold))
         self._thread.finished.connect(self._updateData)
-        self._parametersDock.abortTP.show()
+        if not self._update_dataset.in_memory:
+            self._parametersDock.abortTP.show()
         self._thread.start()
         self._setComputingState(True)
 
@@ -330,7 +336,7 @@ class NoiseRemovalWidget(qt.QMainWindow):
 
     def __abortTP(self):
         self._parametersDock.abortTP.setEnabled(False)
-        self._update_dataset.stop_operation(Operation.TP)
+        self._update_dataset.stop_operation(Operation.THRESHOLD)
 
     def _setComputingState(self, state):
 
@@ -383,6 +389,7 @@ class NoiseRemovalWidget(qt.QMainWindow):
         return self._update_dataset, self.indices, self.bg_indices, self. bg_dataset
 
     def resetStack(self):
+        self._update_dataset = self.dataset
         self.setStack()
 
     def clearStack(self):
@@ -439,6 +446,27 @@ class NoiseRemovalWidget(qt.QMainWindow):
         self._method = method
         self._parametersDock.bsMethodsCB.setCurrentText(method)
 
+    @property
+    def step(self):
+        return self._step
+
+    @step.setter
+    def step(self, step):
+        self._step = step
+        if step is not None:
+            self._parametersDock.step.setText(str(step))
+
+    @property
+    def chunks(self):
+        return self._chunks
+
+    @chunks.setter
+    def chunks(self, chunks):
+        self._chunks = chunks
+        if chunks is not None:
+            self._parametersDock.verticalChunkSize.setText(str(chunks[0]))
+            self._parametersDock.horizontalChunkSize.setText(str(chunks[1]))
+
     @property
     def background(self):
         return self._background
