@@ -26,7 +26,7 @@
 
 __authors__ = ["J. Garriga"]
 __license__ = "MIT"
-__date__ = "08/04/2020"
+__date__ = "05/07/2021"
 
 # import os
 import numpy
@@ -42,19 +42,54 @@ from .operationThread import OperationThread
 from .utils import ChooseDimensionDock
 
 
+class ShiftCorrectionDialog(qt.QDialog):
+    """
+    Dialog with `ShiftCorrectionWidget` as main window and standard buttons.
+    """
+
+    okSignal = qt.Signal()
+
+    def __init__(self, parent=None):
+        qt.QDialog.__init__(self, parent)
+        self.setWindowFlags(qt.Qt.Widget)
+        types = qt.QDialogButtonBox.Ok
+        self._buttons = qt.QDialogButtonBox(parent=self)
+        self._buttons.setStandardButtons(types)
+        self._buttons.setEnabled(False)
+        resetB = self._buttons.addButton(self._buttons.Reset)
+        self.mainWindow = ShiftCorrectionWidget(parent=self)
+        self.mainWindow.setAttribute(qt.Qt.WA_DeleteOnClose)
+        self.setLayout(qt.QVBoxLayout())
+        self.layout().addWidget(self.mainWindow)
+        self.layout().addWidget(self._buttons)
+
+        self._buttons.accepted.connect(self.okSignal.emit)
+        resetB.clicked.connect(self.mainWindow.resetStack)
+        self.mainWindow.computingSignal.connect(self._toggleButton)
+
+    def setDataset(self, dataset, indices=None, bg_indices=None, bg_dataset=None):
+
+        if dataset is not None:
+            self._buttons.setEnabled(True)
+            self.mainWindow.setDataset(dataset, indices, bg_indices, bg_dataset)
+
+    def _toggleButton(self, state):
+        self._buttons.button(qt.QDialogButtonBox.Ok).setEnabled(not state)
+
+
 class ShiftCorrectionWidget(qt.QMainWindow):
     """
     A widget to apply shift correction to a stack of images
     """
-    sigComputed = qt.Signal()
-    sigProgressChanged = qt.Signal(int)
+    computingSignal = qt.Signal(bool)
 
     def __init__(self, parent=None):
         qt.QMainWindow.__init__(self, parent)
 
         self.setWindowFlags(qt.Qt.Widget)
-        self._shift = [0, 0]
-        self.dimension = None
+        self._shift = numpy.array([0, 0])
+        self._filtered_shift = None
+        self._dimension = None
         self._update_dataset = None
         self.indices = None
         self.bg_indices = None
@@ -65,7 +100,7 @@ class ShiftCorrectionWidget(qt.QMainWindow):
 
         self._sv = StackViewMainWindow()
         self._sv.setColormap(Colormap(name=darfix.config.DEFAULT_COLORMAP_NAME,
-                                      normalization=darfix.config.DEFAULT_COLORMAP_NORM))
+                                      normalization="linear"))
         self.setCentralWidget(self._sv)
         self._chooseDimensionDock = ChooseDimensionDock(self)
         spacer1 = qt.QWidget(parent=self)
@@ -82,8 +117,9 @@ class ShiftCorrectionWidget(qt.QMainWindow):
 
         self._inputDock.widget.correctionB.clicked.connect(self.correct)
         self._inputDock.widget.abortB.clicked.connect(self.abort)
+        self._inputDock.widget.dxLE.editingFinished.connect(self._updateShiftValue)
+        self._inputDock.widget.dyLE.editingFinished.connect(self._updateShiftValue)
         self._inputDock.widget._findShiftB.clicked.connect(self._findShift)
-        self._inputDock.widget.checkbox.stateChanged.connect(self._changeDataShiftFlag)
         self._chooseDimensionDock.widget.filterChanged.connect(self._filterStack)
         self._chooseDimensionDock.widget.stateDisabled.connect(self._wholeStack)
 
@@ -97,13 +133,14 @@ class ShiftCorrectionWidget(qt.QMainWindow):
         self.dataset = dataset
         self._update_dataset = dataset
         self.indices = indices
-        self.setStack()
         self.bg_indices = bg_indices
         self.bg_dataset = bg_dataset
         self._inputDock.widget.correctionB.setEnabled(True)
         if len(self.dataset.data.shape) > 3:
             self._chooseDimensionDock.show()
             self._chooseDimensionDock.widget.setDimensions(self._update_dataset.dims)
+        if not self._chooseDimensionDock.widget._checkbox.isChecked():
+            self._wholeStack()
 
     def getDataset(self):
         return self._update_dataset, self.indices, self.bg_indices, self.bg_dataset
@@ -115,37 +152,56 @@ class ShiftCorrectionWidget(qt.QMainWindow):
         """
         dx = self._inputDock.widget.getDx()
         dy = self._inputDock.widget.getDy()
-        self.shift = [dy, dx]
-        dimension = self.dimension if not self._changeDataShiftFlag else None
-        frames = numpy.arange(self.dataset.get_data(indices=self.indices, dimension=dimension).shape[0])
-        self.thread_correction = OperationThread(self, self._update_dataset.apply_shift)
-        self.thread_correction.setArgs(numpy.outer(self.shift, frames), dimension, indices=self.indices)
+        self.shift = numpy.array([dy, dx])
+        if self._filtered_shift is None or self._inputDock.widget.checkbox.isChecked():
+            frames = numpy.arange(self._update_dataset.get_data(indices=self.indices, dimension=self._dimension).shape[0])
+            self.thread_correction = OperationThread(self, self._update_dataset.apply_shift)
+            self.thread_correction.setArgs(numpy.outer(self.shift, frames), self._dimension, indices=self.indices)
+        else:
+            self.thread_correction = OperationThread(self, self._update_dataset.apply_shift_along_dimension)
+            self.thread_correction.setArgs(self._filtered_shift, self._dimension[0], indices=self.indices)
         self.thread_correction.finished.connect(self._updateData)
         self._inputDock.widget.correctionB.setEnabled(False)
         self._inputDock.widget.abortB.show()
         self.thread_correction.start()
+        self.computingSignal.emit(True)
 
     def abort(self):
         self._inputDock.widget.abortB.setEnabled(False)
         self._update_dataset.stop_operation(Operation.SHIFT)
 
+    def resetStack(self):
+        self._update_dataset = self.dataset
+        self.setStack()
+
     def updateProgress(self, progress):
         self.sigProgressChanged.emit(progress)
 
-    def _changeDataShiftFlag(self, state):
-        self._shift2dataset = state
-
     def _findShift(self):
-        self.thread_detection = OperationThread(self, self._update_dataset.find_shift)
+        if self._filtered_shift is not None:
+            self.thread_detection = OperationThread(self, self._update_dataset.find_shift_along_dimension)
+            self.thread_detection.setArgs(self._dimension[0], indices=self.indices)
+        else:
+            self.thread_detection = OperationThread(self, self._update_dataset.find_shift)
+            self.thread_detection.setArgs(self._dimension, indices=self.indices)
         self._inputDock.widget._findShiftB.setEnabled(False)
-        self.thread_detection.setArgs(self.dimension, indices=self.indices)
         self.thread_detection.finished.connect(self._updateShift)
         self.thread_detection.start()
+        self.computingSignal.emit(True)
+
+    def _updateShiftValue(self):
+        if self._filtered_shift is not None:
+            self._filtered_shift[self._dimension[1]] = [self._inputDock.widget.getDy(), self._inputDock.widget.getDx()]
 
     def _updateShift(self):
         self._inputDock.widget._findShiftB.setEnabled(True)
         self.thread_detection.finished.disconnect(self._updateShift)
-        self.shift = self.thread_detection.data[:, 1]
+        if self._filtered_shift is None:
+            self.shift = numpy.round(self.thread_detection.data[:, 1], 5)
+        else:
+            self._filtered_shift = numpy.round(self.thread_detection.data[:, :, 1], 5)
+            self.shift = self._filtered_shift[self._dimension[1][0]]
+        self.computingSignal.emit(False)
 
     def _updateData(self):
         """
@@ -155,11 +211,11 @@ class ShiftCorrectionWidget(qt.QMainWindow):
         self._inputDock.widget.abortB.hide()
         self._inputDock.widget.abortB.setEnabled(True)
         self._inputDock.widget.correctionB.setEnabled(True)
+        self.computingSignal.emit(False)
         if self.thread_correction.data:
             self._update_dataset = self.thread_correction.data
             assert self._update_dataset is not None
             self.setStack(self._update_dataset)
-            self.sigComputed.emit()
         else:
             print("\nCorrection aborted")
 
@@ -173,10 +229,7 @@ class ShiftCorrectionWidget(qt.QMainWindow):
         if dataset is None:
             dataset = self.dataset
         nframe = self._sv.getFrameNumber()
-        if self.indices is None:
-            self._sv.setStack(dataset.get_data() if dataset is not None else None)
-        else:
-            self._sv.setStack(dataset.get_data(self.indices) if dataset is not None else None)
+        self._sv.setStack(dataset.get_data(self.indices, self._dimension) if dataset is not None else None)
         self._sv.setFrameNumber(nframe)
 
     def clearStack(self):
@@ -184,18 +237,26 @@ class ShiftCorrectionWidget(qt.QMainWindow):
         self._inputDock.widget.correctionB.setEnabled(False)
 
     def _filterStack(self, dim=0, val=0):
-        self._inputDock.widget.checkbox.show()
-        self.dimension = [dim, val]
-        data = self.dataset.get_data(self.indices, self.dimension)
+        self._dimension = [dim, val]
+
+        data = self._update_dataset.get_data(self.indices, self._dimension)
+        if self.dataset.dims.ndim == 2:
+            stack_size = self.dataset.dims.get(dim[0]).size
+            reset_shift = self._filtered_shift is None or self._filtered_shift.shape[0] != stack_size
+            self._inputDock.widget.checkbox.show()
+            self._filtered_shift = numpy.zeros((stack_size, 2)) if reset_shift else self._filtered_shift
+            self.shift = self._filtered_shift[val[0]]
         if data.shape[0]:
             self._sv.setStack(data)
         else:
             self._sv.setStack(None)
 
     def _wholeStack(self):
-        self.dimension = None
+        self._dimension = None
+        self._filtered_shift = None
+        self.shift = numpy.array([0, 0])
         self._inputDock.widget.checkbox.hide()
-        self.setStack()
+        self.setStack(self._update_dataset)
 
     def getStack(self):
         """
@@ -205,6 +266,23 @@ class ShiftCorrectionWidget(qt.QMainWindow):
         """
         return self._sv
 
+    def getStackViewColormap(self):
+        """
+        Returns the colormap from the stackView
+
+        :rtype: silx.gui.colors.Colormap
+        """
+        return self._sv.getColormap()
+
+    def setStackViewColormap(self, colormap):
+        """
+        Sets the stackView colormap
+
+        :param colormap: Colormap to set
+        :type colormap: silx.gui.colors.Colormap
+        """
+        self._sv.setColormap(colormap)
+
     @property
     def shift(self):
         return self._shift
@@ -239,8 +317,8 @@ class _InputWidget(qt.QWidget):
         self.correctionB = qt.QPushButton("Correct")
         self.abortB = qt.QPushButton("Abort")
         self.abortB.hide()
-        self.checkbox = qt.QCheckBox("Apply to whole dataset")
-        self.checkbox.setChecked(True)
+        self.checkbox = qt.QCheckBox("Apply only to selected value")
+        self.checkbox.setChecked(False)
         self.checkbox.hide()
 
         self.dxLE.setValidator(qt.QDoubleValidator())
