@@ -26,10 +26,12 @@
 
 __authors__ = ["J. Garriga"]
 __license__ = "MIT"
-__date__ = "08/04/2020"
+__date__ = "22/12/2020"
 
 
+from Orange.widgets.settings import Setting
 from Orange.widgets.widget import OWWidget, Input, Output
+from silx.gui.colors import Colormap
 from darfix.gui.blindSourceSeparationWidget import BSSWidget
 
 
@@ -42,23 +44,39 @@ class BlindSourceSeparationWidgetOW(OWWidget):
     icon = "icons/bss.png"
     want_main_area = False
 
+    # Settings
+    method = Setting(str(), schema_only=True)
+    n_comp = Setting(int(), schema_only=True)
+
     # Inputs
     class Inputs:
         dataset = Input("dataset", tuple)
+        colormap = Input("colormap", Colormap)
 
     # Outputs
     class Outputs:
         dataset = Output("dataset", tuple)
+        colormap = Output("colormap", Colormap)
 
     def __init__(self):
         super().__init__()
 
         self._widget = BSSWidget(parent=self)
+        self._widget.sigComputed.connect(self._updateSettings)
         self.controlArea.layout().addWidget(self._widget)
 
+    def _updateSettings(self, method, n_comp):
+        self.method = method.name
+        self.n_comp = n_comp
+
     @Inputs.dataset
     def setDataset(self, dataset):
         if dataset:
             self._widget.setDataset(*dataset)
             self.open()
         self.Outputs.dataset.send(dataset)
+        self.Outputs.colormap.send(self._widget.getDisplayComponentsWidget().getStackViewColormap())
+
+    @Inputs.colormap
+    def setColormap(self, colormap):
+        self._widget.getDisplayComponentsWidget().setStackViewColormap(colormap)
