@@ -26,13 +26,14 @@
 
 __authors__ = ["J. Garriga"]
 __license__ = "MIT"
-__date__ = "16/04/2020"
+__date__ = "11/12/2020"
 
 
-from Orange.widgets import gui
 from Orange.widgets.settings import Setting
 from Orange.widgets.widget import OWWidget, Input, Output
-from darfix.gui.shiftCorrectionWidget import ShiftCorrectionWidget
+from silx.gui import qt
+from silx.gui.colors import Colormap
+from darfix.gui.shiftCorrectionWidget import ShiftCorrectionDialog
 
 
 class ShiftCorrectionWidgetOW(OWWidget):
@@ -47,21 +48,22 @@ class ShiftCorrectionWidgetOW(OWWidget):
     # Inputs
     class Inputs:
         dataset = Input("dataset", tuple)
+        colormap = Input("colormap", Colormap)
 
     # Outputs
     class Outputs:
         dataset = Output("dataset", tuple)
+        colormap = Output("colormap", Colormap)
 
     # Settings
     shift = Setting(list(), schema_only=True)
 
     def __init__(self):
         super().__init__()
+        qt.QLocale.setDefault(qt.QLocale("en_US"))
 
-        self._widget = ShiftCorrectionWidget(parent=self)
-        self._progressBar = gui.ProgressBar(self, 100)
-        self._widget.sigComputed.connect(self._sendSignal)
-        self._widget.sigProgressChanged.connect(self._updateProgress)
+        self._widget = ShiftCorrectionDialog(parent=self)
+        self._widget.okSignal.connect(self._sendSignal)
         self.controlArea.layout().addWidget(self._widget)
 
         if self.shift:
@@ -77,19 +79,15 @@ class ShiftCorrectionWidgetOW(OWWidget):
             # Emit None
             self.Outputs.dataset.send(dataset)
 
+    @Inputs.colormap
+    def setColormap(self, colormap):
+        self._widget.setStackViewColormap(colormap)
+
     def _sendSignal(self):
         """
         Function to emit the new dataset.
         """
         self.shift = self._widget.shift
-        self.Outputs.dataset.send(self._widget.getDataset())
-        self._progressBar.finish()
+        self.Outputs.dataset.send(self._widget.mainWindow.getDataset())
+        self.Outputs.colormap.send(self._widget.mainWindow.getStackViewColormap())
         self.close()
-
-    def _updateProgress(self, progress):
-        """
-        Updates the progress bar
-
-        :param int progress: the new value that should appear
-        """
-        self._progressBar.widget.progressBarSet(progress)
